/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright© 2020 MikroElektronika d.o.o.
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
 * OR OTHER DEALINGS IN THE SOFTWARE. 
 */

/*!
 * \file
 *
 * \brief This file contains API for Boost 4 Click driver.
 *
 * \addtogroup boost4 Boost 4 Click Driver
 * @{
 */
// ----------------------------------------------------------------------------

#ifndef BOOST4_H
#define BOOST4_H

#include "drv_digital_out.h"
#include "drv_spi_master.h"


// -------------------------------------------------------------- PUBLIC MACROS 
/**
 * \defgroup macros Macros
 * \{
 */

/**
 * \defgroup map_mikrobus MikroBUS
 * \{
 */

#define BOOST4_MAP_MIKROBUS( cfg, mikrobus ) \
   cfg.miso  = MIKROBUS( mikrobus, MIKROBUS_MISO ); \
   cfg.mosi  = MIKROBUS( mikrobus, MIKROBUS_MOSI ); \
   cfg.sck   = MIKROBUS( mikrobus, MIKROBUS_SCK ); \
   cfg.cs    = MIKROBUS( mikrobus, MIKROBUS_CS ); \
   cfg.en = MIKROBUS( mikrobus, MIKROBUS_RST )
/** \} */

/**
 * \defgroup error_code Error Code
 * \{
 */
#define BOOST4_RETVAL  uint8_t

#define BOOST4_OK           0x00
#define BOOST4_INIT_ERROR   0xFF
/** \} */

/**
 * \defgroup dev_cmd Device Commands
 * \{
 */
#define BOOST4_START_CMD  0x70

#define BOOST4_VOUT_MIN   1500 
#define BOOST4_VOUT_2_5   1330
#define BOOST4_VOUT_3     1085
#define BOOST4_VOUT_3_5   840
#define BOOST4_VOUT_4     595
#define BOOST4_VOUT_4_5   350
#define BOOST4_VOUT_5     105
#define BOOST4_VOUT_MAX   0

#define BOOST4_12_BIT     0x0FFF
#define BOOST4_ENABLE     1
#define BOOST4_DISABLE    0
/** \} */

/** \} */ // End group macro 
// --------------------------------------------------------------- PUBLIC TYPES
/**
 * \defgroup type Types
 * \{
 */

/**
 * @brief Click ctx object definition.
 */
typedef struct
{
    // Output pins 

    digital_out_t en;
    digital_out_t cs;

    // Modules 

    spi_master_t spi;
    pin_name_t chip_select;

} boost4_t;

/**
 * @brief Click configuration structure definition.
 */
typedef struct
{
    // Communication gpio pins 

    pin_name_t miso;
    pin_name_t mosi;
    pin_name_t sck;
    pin_name_t cs;

    // Additional gpio pins 

    pin_name_t en;


    // static variable 

    uint32_t spi_speed;
    spi_master_mode_t spi_mode;
    spi_master_chip_select_polarity_t cs_polarity;

} boost4_cfg_t;

/** \} */ // End types group

// ----------------------------------------------- PUBLIC FUNCTION DECLARATIONS

/**
 * \defgroup public_function Public function
 * \{
 */
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief Config Object Initialization function.
 *
 * @param cfg  Click configuration structure.
 *
 * @description This function initializes click configuration structure 
 * to init state.
 * @note All used pins will be set to unconnected state.
 */
void boost4_cfg_setup ( boost4_cfg_t *cfg );

/**
 * @brief Initialization function.
 * @param boost4 Click object.
 * @param cfg Click configuration structure.
 * 
 * @description This function initializes all necessary pins and peripherals 
 * used for this click.
 */
BOOST4_RETVAL boost4_init ( boost4_t *ctx, boost4_cfg_t *cfg );

/**
 * @brief Enable device function 
 *
 * @param ctx          Click object.
 * @param   state  0 - device disabled / 1 - device enabled.
 *
 * @description Function is used to enabled or disabled the device.
 */
void boost4_enable ( boost4_t *ctx, uint8_t state );

/**
 * @brief Set output voltage function
 *
 * @param ctx  Click object.
 * @param value 12-bit data that defines output voltage.
 *
 * @description Function set output voltage by write 12-bit data to the register
 * on the TPS61230A High Efficiency Step-Up Converter of Boost 4 Click.
 */
void boost4_set_out_voltage ( boost4_t *ctx, uint16_t value );

#ifdef __cplusplus
}
#endif
#endif  // _BOOST4_H_

/** \} */ // End public_function group
/// \}    // End click Driver group  
/*! @} */
// ------------------------------------------------------------------------- END
